/*
 *  echoback.c:
 *	Simple test program to test the wiringPi functions
 *	echoback test
 */
 
#include <wiringPi.h>
#include <wiringPiSPI.h>
 
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

#include "wizchip_conf.h"
#include "loopback.h"

///////////////////////////////////////
// Debugging Message Printout enable //
///////////////////////////////////////
#define _MAIN_DEBUG_

///////////////////////////
// Demo Firmware Version //
///////////////////////////
#define VER_H		1
#define VER_L		00

//////////////////////////////////////////////////
// Socket & Port number definition for Examples //
//////////////////////////////////////////////////
#define SOCK_TCPS       0
#define SOCK_UDPS       1
#define PORT_TCPS	5100
#define PORT_UDPS       3000

////////////////////////////////////////////////
// Shared Buffer Definition for LOOPBACK TEST //
////////////////////////////////////////////////
uint8_t gDATABUF[DATA_BUF_SIZE];

///////////////////////////
// Network Configuration //
///////////////////////////
wiz_NetInfo gWIZNETINFO = { .mac = {0x00, 0x08, 0xdc, 0x01, 0x01, 0x01},
			    .ip = {192, 168, 0, 101},
			    .sn = {255, 255, 255, 0},
			    .gw = {192, 168, 0, 1},
			    .dns = {8, 8, 8, 8},
			    .dhcp = NETINFO_STATIC };

// For TCP client echechok examples; destination network info
uint8_t destip[4] = {192, 168, 0, 9};
uint16_t destport = 5000;

#define RDY		1
#define CS		4
#define RST		5
#define PWR		6
#define CHANNEL		0
uint8_t ret;
unsigned char buffer[100];
uint8_t g_flag;

void wizchip_select(void);
void wizchip_deselect(void);
uint8_t wizchip_read();
void wizchip_write(uint8_t wb);
void Net_Conf(wiz_NetInfo netinfo);
void Display_Net_Conf();

int main( void )
{
	int fd;
	uint8_t memsize[2][8] = {{2,2,2,2,2,2,2,2}, {2,2,2,2,2,2,2,2}};
	uint8_t tmp;
	int32_t echoback_ret;

	printf( "\r\nRaspberry Pi wiringPi SPI test program\r\n" );
	
	if ( wiringPiSetup() == -1 )
		exit( 1 );

#if !defined(WIZ550ioWITH_MACADDRESS)
	//W5500 reset
	pinMode(RST, OUTPUT);
	digitalWrite(RST, 0);
	delay(1);
	digitalWrite(RST, 1);
	delay(100);
#endif

#if defined(WIZ550ioWITH_MACADDRESS)
	//Need to control PWR before checking RDY
	//Check RDY
	pinMode(RDY, INPUT);
	do {
		delay(100);
	} while(digitalRead(RDY) == 0);
#endif

	//SPI init
#if 0
	//unsigned char buffer[100];
 
	pinMode(CS, OUTPUT);
	fd = wiringPiSPISetup(CHANNEL, 1000000);
	printf( "fd:%d\n", fd);

	//digitalWrite(CS, 0);
	//buffer[0] = 0x76;
	//ret = wiringPiSPIDataRW(CHANNEL, buffer, 1);
	//printf("SPI read:0x%02x 0x%02x\r\n", ret, buffer[0]);
	//digitalWrite(CS, 1);
	int i;
	for(i=0; i<20; i++)
	{
		digitalWrite(CS, 0);
		wizchip_write(0x76+i);
		digitalWrite(CS, 1);
		delay(50);
	}
#else
	pinMode(CS, OUTPUT);
	fd = wiringPiSPISetup(CHANNEL, 8000000);
	//printf( "fd:%d\n", fd);

	//W5500 init
	reg_wizchip_cs_cbfunc(wizchip_select, wizchip_deselect);
	reg_wizchip_spi_cbfunc(wizchip_read, wizchip_write);

#if !defined(WIZ550ioWITH_MACADDRESS)
	if(ctlwizchip(CW_INIT_WIZCHIP, (void*) memsize) == -1)
	{
		printf("WIZCHIP Initialized fail.\r\n");
		while(1);
	}

	do
	{
		if(ctlwizchip(CW_GET_PHYLINK, (void*)&tmp) == -1)
		{
			printf("Unknown PHY Link status.\r\n");
			return 0;
		}
	} while (tmp == PHY_LINK_OFF);
#endif

	//Network init
	Net_Conf(gWIZNETINFO);

#ifdef _MAIN_DEBUG_
	uint8_t tmpstr[6] = {0,};

	ctlwizchip(CW_GET_ID,(void*)tmpstr);

	printf("\r\n=======================================\r\n");
	printf(" WIZnet %s WIZ850io Demos v%d.%.2d\r\n", tmpstr, VER_H, VER_L);
	printf("=======================================\r\n");
	printf(">> W5500 based echoback Example\r\n");
	printf("=======================================\r\n");

	Display_Net_Conf(); // Print out the network information to serial terminal
#endif
#endif

#if 0
	wiz_NetTimeout gtimeout, stimeout;
	stimeout.retry_cnt = 8;
	stimeout.time_100us = 2000;
	wizchip_settimeout(&stimeout);
	wizchip_gettimeout(&gtimeout);
	//printf("gtimeout.retry_cnt:%d time_100us:%d\r\n", gtimeout.retry_cnt, gtimeout.time_100us);
#endif

	g_flag = 0;

	while ( 1 )
	{
		/* echoback Test: TCP Server and UDP */
		// Test for Ethernet data transfer validation
		{
			loopback_tcps(SOCK_TCPS, gDATABUF, PORT_TCPS);
			//loopback_udps(SOCK_UDPS, gDATABUF, PORT_UDPS);
			//echoback_ret = loopback_tcpc(SOCK_TCPS, gDATABUF, destip, destport);

			//if(echoback_ret < 0) printf("echoback ret: %ld\r\n", echoback_ret); // TCP Socket Error code
		}
	}
 
	return(0);
}

void wizchip_select(void)
{
	//printf(">> SPI select\r\n");
	digitalWrite(CS, 0);
}

void wizchip_deselect(void)
{
	//printf("<< SPI deselect\r\n");
	digitalWrite(CS, 1);
}

uint8_t wizchip_read()
{
	uint8_t rb;
	
	ret = wiringPiSPIDataRW(CHANNEL, &rb,1);
	//printf("SPI read:0x%02x\r\n", rb);

	return rb;
}

void wizchip_write(uint8_t wb)
{
	//printf("SPI write before:0x%02x\r\n", wb);
	ret = wiringPiSPIDataRW(CHANNEL, &wb, 1);
}

void Net_Conf(wiz_NetInfo netinfo)
{
	//printf("\r\nMAC: %02X:%02X:%02X:%02X:%02X:%02X\r\n", netinfo.mac[0], netinfo.mac[1], netinfo.mac[2], netinfo.mac[3], netinfo.mac[4], netinfo.mac[5]);
	//printf("IP: %d.%d.%d.%d\r\n", netinfo.ip[0], netinfo.ip[1], netinfo.ip[2], netinfo.ip[3]);
	//printf("GW: %d.%d.%d.%d\r\n", netinfo.gw[0], netinfo.gw[1], netinfo.gw[2], netinfo.gw[3]);
	//printf("SN: %d.%d.%d.%d\r\n", netinfo.sn[0], netinfo.sn[1], netinfo.sn[2], netinfo.sn[3]);
	//printf("DNS: %d.%d.%d.%d\r\n", netinfo.dns[0], netinfo.dns[1], netinfo.dns[2], netinfo.dns[3]);
	ctlnetwork(CN_SET_NETINFO, (void*) &netinfo);
}

void Display_Net_Conf()
{
	uint8_t tmpstr[6] = {0,};
	wiz_NetInfo WIZNETINFO;

	ctlnetwork(CN_GET_NETINFO, (void*) &WIZNETINFO);
	ctlwizchip(CW_GET_ID,(void*)tmpstr);

	// Display Network Information
	if(WIZNETINFO.dhcp == NETINFO_DHCP) printf("\r\n===== %s NET CONF : DHCP =====\r\n",(char*)tmpstr);
		else printf("\r\n===== %s NET CONF : Static =====\r\n",(char*)tmpstr);

	printf("\r\nMAC: %02X:%02X:%02X:%02X:%02X:%02X\r\n", WIZNETINFO.mac[0], WIZNETINFO.mac[1], WIZNETINFO.mac[2], WIZNETINFO.mac[3], WIZNETINFO.mac[4], WIZNETINFO.mac[5]);
	printf("IP: %d.%d.%d.%d\r\n", WIZNETINFO.ip[0], WIZNETINFO.ip[1], WIZNETINFO.ip[2], WIZNETINFO.ip[3]);
	printf("GW: %d.%d.%d.%d\r\n", WIZNETINFO.gw[0], WIZNETINFO.gw[1], WIZNETINFO.gw[2], WIZNETINFO.gw[3]);
	printf("SN: %d.%d.%d.%d\r\n", WIZNETINFO.sn[0], WIZNETINFO.sn[1], WIZNETINFO.sn[2], WIZNETINFO.sn[3]);
	printf("DNS: %d.%d.%d.%d\r\n", WIZNETINFO.dns[0], WIZNETINFO.dns[1], WIZNETINFO.dns[2], WIZNETINFO.dns[3]);
}

